#!/bin/bash

###############################################################################
# semestrální práce z předmětu Y36SPS
#
# AUTOMATICKÝ GENERÁTOR KONFIGURACE DÉMONA NRPE A NAGIOSU
# Petr Jelínek (jelinp8)
#
###############################################################################
 VERSION="0.6"
###############################################################################

# CONFIGURATION

# OUTPUT FILES
#NRPE_CFG_FILE="nrpe.cfg"
NRPE_CFG_FILE="/etc/nagios/gen_nrpe.cfg"
NAGIOS_TEMPLATE="nagios_template.cfg"
# NAGIOS_TEMPLATE="/etc/nagios/objects/gen_y36sps.cfg"

# NAGIOS TEMPLATE CONFIGURATION
USE="local-service" # name of service template to use
HOST_NAME=`hostname`

# path to nagios plugins
NRPE_PLUGINS="/usr/lib/nagios/plugins"

# TEST CONFIGURATION
DNS_TEST_HOST="google.com"
NTP_TEST_SERVER="cz.pool.ntp.org"
PING_TEST_HOST="google.com"

CHECK_DISK_W="10%"
CHECK_DISK_C="5%"

CHECK_LOAD_WLOAD1=0.85
CHECK_LOAD_WLOAD5=0.75
CHECK_LOAD_WLOAD15=0.65
CHECK_LOAD_CLOAD1=0.95
CHECK_LOAD_CLOAD5=0.85
CHECK_LOAD_CLOAD15=0.75

CHECK_NTP_TIME_W=15
CHECK_NTP_TIME_C=60

CHECK_PING_W="100,20%"
CHECK_PING_C="1000,50%"

CHECK_PROCS_W=":200"
CHECK_PROCS_C=":1000"

CHECK_SWAP_W="25%"
CHECK_SWAP_C="10%"

CHECK_USERS_W="10"
CHECK_USERS_C="20"

###############################################################################

# FUNCTIONS

function add_to_nrpe {
  echo -e "add to \"$NRPE_CFG_FILE\": $1"
  echo -e "$1" >> "$NRPE_CFG_FILE"
}

function add_to_nagios {
  RECORD="define service {\n"
  RECORD="$RECORD\tuse\t\t\t$USE\n"
  RECORD="$RECORD\thost_name\t\t$HOST_NAME\n"
  RECORD="$RECORD\tservice_description\t$2\n"
  RECORD="$RECORD\tcheck_command\t\tcheck_nrpe!$1\n}"

  echo -e "add to \"$NAGIOS_TEMPLATE\":\n$RECORD"
  echo -e "$RECORD\n" >> "$NAGIOS_TEMPLATE"
}

###############################################################################

echo "# NRPE configuration file" > "$NRPE_CFG_FILE"
echo "# created by nrpecfg.sh $VERSION" >> "$NRPE_CFG_FILE"
echo "" >> "$NRPE_CFG_FILE"

echo "# nagios configuration file templates" > "$NAGIOS_TEMPLATE"
echo "# created by nrpecfg.sh $VERSION" >> "$NAGIOS_TEMPLATE"
echo "" >> "$NAGIOS_TEMPLATE"

echo -e "define host {" >> "$NAGIOS_TEMPLATE"
echo -e "\tname\t\t\t$HOST_NAME" >> "$NAGIOS_TEMPLATE"
echo -e "\thost_name\t\t$HOST_NAME" >> "$NAGIOS_TEMPLATE"
echo -e "\tmax_check_attempts\t10$HOST_NAME" >> "$NAGIOS_TEMPLATE"
echo -e "}" >> "$NAGIOS_TEMPLATE"

echo "" >> "$NAGIOS_TEMPLATE"
echo "" >> "$NAGIOS_TEMPLATE"

###############################################################################
###############################################################################

# CHECK_APT
# This plugin checks for software updates on systems that use package
# management systems based on the apt-get(8) command found in Debian GNU/Linux

echo "check_apt..."
if which apt-get 2>/dev/null; then
  add_to_nrpe "command[check_apt]=$NRPE_PLUGINS/check_apt"
  add_to_nagios "check_apt" "APT"
else
  echo "...apt-get not found"
fi
echo ""

###############################################################################

# CHECK_DHCP
# This plugin tests the availability of DHCP servers on a network.

echo "check_dhcp..."
add_to_nrpe "command[check_dhcp]=$NRPE_PLUGINS/check_dhcp"
add_to_nagios "check_dhcp" "DHCP"
for I in `ifconfig | grep Ethernet | sed 's/\s.*//'`; do
  add_to_nrpe "command[check_dhcp_$I]=$NRPE_PLUGINS/check_dhcp --interface=\"$I\""
  add_to_nagios "check_dhcp_$I" "DHCP [$I]"
done
echo ""

###############################################################################

# CHECK_DIG
# This plugin test the DNS service on the specified host using dig

echo "check_dig..."
add_to_nrpe "command[check_dig]=$NRPE_PLUGINS/check_dig --query_address=\"$DNS_TEST_HOST\""
add_to_nagios "check_dig" "DNS via DIG [$DNS_TEST_HOST]"
echo ""

###############################################################################

# CHECK_DISK
# This plugin checks the amount of used disk space on a mounted file system and
# generates an alert if free space is less than one of the threshold values

echo "check_disk..."
add_to_nrpe "command[check_all_disks]=$NRPE_PLUGINS/check_disk --iwarning=\"$CHECK_DISK_W\" --icritical=\"$CHECK_DISK_C\""
add_to_nagios "check_all_disks" "used disk space [all disks]"
for I in `df | grep '/dev/' | sed 's/.*\s//'`; do
  SERVICE_NAME=`[ $I == "/" ] && echo "root" || echo $I | sed 's/.//;y/\//-/'`
  add_to_nrpe "command[check_disk_$SERVICE_NAME]=$NRPE_PLUGINS/check_disk --iwarning=\"$CHECK_DISK_W\" --icritical=\"$CHECK_DISK_C\" --path=\"$I\""
  add_to_nagios "check_disk_$SERVICE_NAME" "used disk space [$I]"
done
echo ""

###############################################################################

# CHECK_DNS
# This plugin uses the nslookup program to obtain the IP address for the given
# host/domain query.
# An optional DNS server to use may be specified.
# If no DNS server is specified, the default server(s) specified in
# /etc/resolv.conf will be used.

echo "check_dns..."
add_to_nrpe "command[check_dns]=$NRPE_PLUGINS/check_dns --hostname=\"$DNS_TEST_HOST\""
add_to_nagios "check_dns" "DNS [$DNS_TEST_HOST]"
echo ""

###############################################################################

# CHECK_IDE_SMART
# This plugin checks a local hard drive with the (Linux specific) SMART
# interface [http://smartlinux.sourceforge.net/smart/index.php].

echo "check_ide_smart..."
for I in `find /dev -type b | egrep '/(hd|sd)[a-z]$' | sort`; do
  SERVICE_NAME=`echo $I | sed 's/.//;y/\//-/'`
  add_to_nrpe "command[check_ide_smart_$SERVICE_NAME]=$NRPE_PLUGINS/check_ide_smart --nagios --device=\"$I\""
  add_to_nagios "check_ide_smart_$SERVICE_NAME" "IDE SMART [$I]"
done
echo ""

###############################################################################

# CHECK_LOAD
# This plugin tests the current system load average.

echo "check_load..."
add_to_nrpe "command[check_load]=$NRPE_PLUGINS/check_load --warning=\"$CHECK_LOAD_WLOAD1,$CHECK_LOAD_WLOAD5,$CHECK_LOAD_WLOAD15\" --critical=\"$CHECK_LOAD_CLOAD1,$CHECK_LOAD_CLOAD5,$CHECK_LOAD_CLOAD15\""
add_to_nagios "check_load" "load"
echo ""

###############################################################################

# CHECK_NTP_TIME
# This plugin checks the clock offset with the ntp server

echo "check_ntp_time..."
add_to_nrpe "command[check_ntp_time]=$NRPE_PLUGINS/check_ntp_time --hostname=\"$NTP_TEST_SERVER\" --warning=\"$CHECK_NTP_TIME_W\" --critical=\"$CHECK_NTP_TIME_C\""
add_to_nagios "check_ntp_time" "clock offset [$NTP_TEST_SERVER]"
echo ""

###############################################################################

# CHECK_PING
# Use ping to check connection statistics for a remote host.

echo "check_ping..."
add_to_nrpe "command[check_ping]=$NRPE_PLUGINS/check_ping --hostname=\"$PING_TEST_HOST\" --warning=\"$CHECK_PING_W\" --critical=\"$CHECK_PING_C\""
add_to_nagios "check_ping" "ping [$PING_TEST_HOST]"
echo ""

###############################################################################

# CHECK_PROCS
# Checks all processes and generates WARNING or CRITICAL states if the
# specified metric is outside the required threshold ranges. The metric
# defaults to number of processes. Search filters can be applied to limit the
# processes to check. 

echo "check_procs..."
add_to_nrpe "command[check_procs]=$NRPE_PLUGINS/check_procs --warning=\"$CHECK_PROCS_W\" --critical=\"$CHECK_PROCS_C\""
add_to_nagios "check_procs" "processes"
echo ""

###############################################################################

# CHECK_SENSORS
# This plugin checks hardware status using the lm_sensors package.

echo "check_sensors..."
if which sensors 2>/dev/null; then
  add_to_nrpe "command[check_sensors]=$NRPE_PLUGINS/check_sensors"
  add_to_nagios "check_sensors" "sensors"
else
  echo "...lmsensors not found"
fi
echo ""

###############################################################################

# CHECK_SWAP
# Check swap space on local machine.

echo "check_swap..."
add_to_nrpe "command[check_swap]=$NRPE_PLUGINS/check_swap --warning=\"$CHECK_SWAP_W\" --critical=\"$CHECK_SWAP_C\""
add_to_nagios "check_swap" "swap"
echo ""

###############################################################################

# CHECK_USERS
# This plugin checks the number of users currently logged in on the local
# system and generates an error if the number exceeds the thresholds specified.

echo "check_users..."
add_to_nrpe "command[check_users]=$NRPE_PLUGINS/check_users --warning=\"$CHECK_USERS_W\" --critical=\"$CHECK_USERS_C\""
add_to_nagios "check_users" "users"
echo ""

###############################################################################

# CHECK_ZYPPER
# This plugin checks for software updates on systems that use package
# management systems based on the zypper command found in openSUSE.

echo "check_zypper..."
if which zypper 2>/dev/null; then
  add_to_nrpe "command[check_zypper]=$NRPE_PLUGINS/check_zypper"
  add_to_nagios "check_zypper" "zypper"
else
  echo "...zypper not found"
fi
echo ""

###############################################################################
###############################################################################
